// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Use case data
class OmnitestUseCase {
  /// Specific description
  String description;

  /// HTTP client that is used when bootstrapping the module
  http.BaseClient httpClient;

  /// Chain of messages that are sequentially when this use cases is tested
  List<OmnitestMessage> messageChain = new List<OmnitestMessage>();

  /// Load data from a YAML map.
  OmnitestUseCase.fromYaml(
      YamlMap yamlData, Map<String, OmnibuildMessage> messageSpec) {
    // Read description.
    description =
        yamlData.containsKey('description') ? yamlData['description'] : '';

    // Construct HTTP client.
    httpClient = yamlData.containsKey('http-client')
        ? _getHttpClient(yamlData['http-client'])
        : new http.IOClient();

    // Loop through messages.
    if (yamlData.containsKey('messages') && yamlData['messages'] is YamlList) {
      yamlData['messages'].forEach((YamlMap yamlData) {
        messageChain.add(new OmnitestMessage.fromYaml(yamlData, messageSpec));
      });
    }
  }
}

/// Single test file data structure
class OmnitestFile {
  /// General description
  String description;

  /// Message details per omnibusId (parsed from the message-spec list)
  Map<String, OmnibuildMessage> messageSpec =
      new Map<String, OmnibuildMessage>();

  /// List of use cases that are defined in this file
  List<OmnitestUseCase> useCases = new List<OmnitestUseCase>();

  /// Load data from a YAML map.
  Future parseYaml(String yamlString) async {
    var yamlData = loadYaml(yamlString);

    // Read description.
    description =
        yamlData.containsKey('description') ? yamlData['description'] : '';

    // Load own package name.
    String thisPackage =
        loadYaml(new File('./pubspec.yaml').readAsStringSync())['name'];

    // Parse message spec.
    var pathMatcher = new RegExp(r'.v(\d+).yaml$');
    var packageMatcher = new RegExp(r'package:([^/]+)\/');
    if (yamlData.containsKey('message-spec') &&
        yamlData['message-spec'] is YamlList) {
      for (var uri in yamlData['message-spec']) {
        var r = pathMatcher.firstMatch(uri);

        // Only proceed if the file URI meets the naming requirements.
        // (name.v#.yaml)
        if (r != null) {
          // Resolve package name.
          var packageName = thisPackage;
          if (uri.startsWith('package:')) {
            var pr = packageMatcher.firstMatch(uri);
            packageName = pr != null ? pr.group(1) : packageName;
          }

          // Parse resource.
          var resource = new Resource(uri);
          var yamlString = await resource.readAsString();
          var spec = new OmnibuildSpecFile.fromYaml(
              yamlString, int.parse(r.group(1)), packageName);

          // Insert all messages into messageSpec.
          spec.messages.forEach((OmnibuildMessage message) {
            messageSpec.putIfAbsent(message.omnibusId, () => message);
          });
        } else {
          // Throw error.
          throw new OmnitestException.yaml(
              '$uri does not meet the naming convention (name.v#.yaml).');
        }
      }
    }

    // Check if the message spec contains any reserved names.
    if (_reservedKeys
        .toSet()
        .intersection(messageSpec.keys.toSet())
        .isNotEmpty) {
      throw new OmnitestException.yaml(
          '$_reservedKeys can not be used as omnibusId.');
    }

    // Loop through use cases.
    if (yamlData.containsKey('use-cases') &&
        yamlData['use-cases'] is YamlList) {
      yamlData['use-cases'].forEach((YamlMap useCase) {
        useCases.add(new OmnitestUseCase.fromYaml(useCase, messageSpec));
      });
    }
  }
}
